"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ControllerType_1 = __importDefault(require("../../enums/ControllerType"));
const Focuser_1 = __importDefault(require("../focuser/Focuser"));
const SelectorKeyboard_1 = __importDefault(require("./SelectorKeyboard"));
const SelectorQuest2_1 = __importDefault(require("./SelectorQuest2"));
function Selector() {
    return [...(0, Focuser_1.default)(), SelectorMixIn];
}
exports.default = Selector;
function SelectorMixIn(Base) {
    let CoreBase = SelectorCore(Base);
    switch (Base.controllerType) {
        case ControllerType_1.default.KEYBOARD:
            return (0, SelectorKeyboard_1.default)(CoreBase);
        case ControllerType_1.default.QUEST2:
            return (0, SelectorQuest2_1.default)(CoreBase);
    }
    return CoreBase;
}
function SelectorCore(Base) {
    class SelectorCoreMixIn extends Base {
        defaults() {
            return Object.assign(Object.assign({}, super.defaults()), { selected: [], selectorTargets: [] });
        }
        update(p) {
            this.selectorTargets = this.project.scene.all
                .map(m => m)
                .filter(m => !!m.isSelectable);
            super.update(p);
        }
        select(v) {
            this.selected.push(v);
            v.select();
            this.onChangeSelected(this.selected);
        }
        deselect(v) {
            this.selected = this.selected.filter(s => s !== v);
            v.deselect();
            this.onChangeSelected(this.selected);
        }
        deselectAll() {
            this.selected.forEach(i => i.deselect());
            this.selected = [];
            this.onChangeSelected(this.selected);
        }
        onChangeSelected(v) { }
    }
    return SelectorCoreMixIn;
}
//# sourceMappingURL=Selector.js.map