"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const GrabActionType_1 = __importDefault(require("../enums/GrabActionType"));
const PlayState_1 = __importDefault(require("../enums/PlayState"));
const RenderMode_1 = __importDefault(require("../enums/RenderMode"));
const setVector_1 = __importDefault(require("../utils/setVector"));
const XRClass_1 = __importDefault(require("./XRClass"));
const XRPlayer_1 = __importDefault(require("./XRPlayer"));
const XRViewer_1 = __importDefault(require("./XRViewer"));
class XRProject extends (0, XRClass_1.default)() {
    get state() {
        return this._state;
    }
    setPlayState(v) {
        this._state.play = v;
    }
    setGrabActionState(v) {
        this._state.grabAction = v;
    }
    setGodModeState(v) {
        this._state.godMode = v;
    }
    setScene(S) {
        this.Scene = S;
        this.start();
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { clock: new three_1.Clock(), _state: {
                play: PlayState_1.default.STOPPED,
                grabAction: GrabActionType_1.default.MOVE,
                godMode: false,
            }, viewers: [] });
    }
    init() {
        this._endXR = this.endXR.bind(this);
        this._tock = this.tock.bind(this);
        super.init();
    }
    initViewer(el, dims) {
        this.viewer = new XRViewer_1.default({ project: this, el, dims });
        this.play();
    }
    setParam(param, value) {
        this.scene.setParam(param, value);
    }
    start() {
        var _a, _b, _c, _d;
        try {
            (_a = this.viewer) === null || _a === void 0 ? void 0 : _a.destroy();
            (_b = this.scene) === null || _b === void 0 ? void 0 : _b.destroy();
            (_c = this.player) === null || _c === void 0 ? void 0 : _c.destroy();
        }
        catch (e) { }
        this.scene = new this.Scene();
        this.player = new XRPlayer_1.default({ project: this });
        this.player.setRenderMode(RenderMode_1.default.DESKTOP);
        (0, setVector_1.default)(this.player.position, this.scene.playerInitPosition);
        (0, setVector_1.default)(this.player.rotation, this.scene.playerInitRotation);
        this.scene.addObject(this.player);
        (_d = this.viewer) === null || _d === void 0 ? void 0 : _d.init();
        this.clock.start();
        this._state.play = PlayState_1.default.PLAYING;
        this.untick();
        this.tick();
    }
    stop() {
        this.pause();
        this.clock.stop();
        this._state.play = PlayState_1.default.STOPPED;
        this.scene.destroy();
    }
    pause() {
        this._state.play = PlayState_1.default.PAUSED;
    }
    play() {
        this._state.play = PlayState_1.default.PLAYING;
        this.tick();
    }
    startXR() {
        this.pause();
        navigator['xr']
            .requestSession('immersive-vr', {
            optionalFeatures: ['local-floor', 'bounded-floor', 'hand-tracking', 'layers'],
        })
            .then(s => {
            this.session = s;
            this.player.setRenderMode(RenderMode_1.default.XR);
            s.addEventListener('end', this._endXR);
            return this.viewer.setXRSession(s);
        })
            .then(() => this.play());
    }
    endXR() {
        this.pause();
        this.session.removeEventListener('end', this._endXR);
        this.player.setRenderMode(RenderMode_1.default.DESKTOP);
        this.session = undefined;
        void this.viewer.setXRSession();
        this.play();
    }
    destroy() {
        var _a, _b, _c;
        this.stop();
        (_a = this.session) === null || _a === void 0 ? void 0 : _a.removeEventListener('end', this._endXR);
        (_b = this.viewer) === null || _b === void 0 ? void 0 : _b.destroy();
        (_c = this.player) === null || _c === void 0 ? void 0 : _c.destroy();
    }
    tick() {
        if (this.session) {
            this.session.requestAnimationFrame(this._tock);
        }
        else {
            window.requestAnimationFrame(this._tock);
        }
    }
    untick() {
        if (this.session) {
            this.session.cancelAnimationFrame(this._tock);
        }
        else {
            window.cancelAnimationFrame(this._tock);
        }
    }
    tock(_, frame) {
        var _a, _b, _c;
        try {
            if (this._state.play === PlayState_1.default.STOPPED)
                return;
            const p = {
                delta: this.clock.getDelta(),
                time: this.clock.getElapsedTime(),
                frame,
            };
            if (this._state.play !== PlayState_1.default.PAUSED)
                (_a = this.scene) === null || _a === void 0 ? void 0 : _a.update(p);
            (_b = this.player) === null || _b === void 0 ? void 0 : _b.update(p);
            (_c = this.viewer) === null || _c === void 0 ? void 0 : _c.update(p);
        }
        catch (e) {
            console.error('Error in XRProject.tock', e);
        }
        this.tick();
    }
}
exports.default = XRProject;
//# sourceMappingURL=XRProject.js.map