"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const three_1 = require("three");
const Plane_1 = __importDefault(require("../objects/Plane"));
class PlayerConsole extends Plane_1.default {
    constructor() {
        super(...arguments);
        this.message = '';
    }
    defaults() {
        return Object.assign(Object.assign({}, super.defaults()), { paneSize: new three_1.Vector2(2, 2), fontSizePx: 50, fontSizeWorld: 0.15, textColor: 0x444444, visible: false });
    }
    init() {
        super.init();
        this.canvas = document.createElement('canvas');
        this.ctx = this.canvas.getContext('2d');
        this.texture = new three_1.Texture(this.canvas);
        this.texture.minFilter = three_1.LinearFilter;
        const worldToPx = this.fontSizeWorld / this.fontSizePx;
        this.canvas.width = this.paneSize.x / worldToPx;
        this.canvas.height = this.paneSize.y / worldToPx;
        this.ctx.textAlign = 'center';
        this.ctx.textBaseline = 'middle';
        this.ctx.font = `${this.fontSizePx}px sans-serif`;
        this.clearScreen();
    }
    show() {
        this.visible = true;
    }
    hide() {
        this.visible = false;
    }
    logVector3(v) {
        this.log([Math.round(v.x * 10) / 10, Math.round(v.y * 10) / 10, Math.round(v.z * 10) / 10].join(', '));
    }
    log(message) {
        const worldToPx = this.fontSizeWorld / this.fontSizePx;
        this.clearScreen();
        const words = message.split(' ');
        const maxCols = this.paneSize.x / (this.fontSizeWorld * 0.6);
        const maxRows = this.paneSize.y / this.fontSizeWorld;
        const rows = [];
        let currentRow = '';
        words.forEach(word => {
            if (currentRow && currentRow.length + word.length > maxCols) {
                rows.push(currentRow);
                currentRow = word;
            }
            else {
                currentRow += ' ' + word;
            }
        });
        rows.push(currentRow);
        this.ctx.fillStyle = '#' + new three_1.Color(this.textColor).getHexString();
        rows.forEach((row, idx) => {
            this.ctx.fillText(row.trim(), this.paneSize.x / 2 / worldToPx, (idx + 1) * this.fontSizePx);
        });
        this.texture.needsUpdate = true;
        this.material.map = this.texture;
    }
    clearScreen() {
        this.ctx.fillStyle = '#' + this.material.color.getHexString();
        this.ctx.fillRect(0, 0, this.canvas.width, this.canvas.height);
    }
}
exports.default = PlayerConsole;
//# sourceMappingURL=PlayerConsole.js.map